//+------------------------------------------------------------------+
//|                                                MACDPredictor.mq4 |
//|               Copyright  2011, Norg, based on Joe DiNapoli idea |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Norg, based on Joe DiNapoli idea"
#property link      "http://www.oxybeauty.com/"
 
#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Red 

/*extern double fastEMA=8.3896; // DiNapoli 8, standard 12
extern double slowEMA=17.5185; // DiNapoli 13, standard 26
extern double macdEMA=9.0503; // DiNapoli 9, standard 9
*/
double fastEMA=8.3896; // DiNapoli 8, standard 12
 double slowEMA=17.5185; // DiNapoli 13, standard 26
 double macdEMA=9.0503; // DiNapoli 9, standard 9

//---- DiNapoli constants
int shiftBarsToFuture = 1 ; // shifts 1 bar into the future
double alpha = 0.0 ;
double beta = 0.0 ;
double gamma_fast = 0.0 ;
double gamma_slow = 0.0 ;
double gamma_macd = 0.0 ;

//---- indicator buffers
double ExtRedBuffer[];
 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- line shifts when drawing
   SetIndexShift(0, shiftBarsToFuture);
//---- indicator buffers mapping
   SetIndexBuffer(0,ExtRedBuffer);
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
//---- index labels
   SetIndexLabel(0,"MACD Predictor");
//---- calculating DiNapoli constants (they only depend on EMA periods)
   gamma_fast = 2.0/(fastEMA+1);
   gamma_slow = 2.0/(slowEMA+1);
   gamma_macd = 2.0/(macdEMA+1);
   alpha = gamma_fast - gamma_slow;
   beta = gamma_macd;

//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| DiNapoli MACD Predictor                                          |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   double current_fastEMA, current_slowEMA, func_curr_EMA, func_curr_sig ;
   int counted_bars=IndicatorCounted();
//---- check for possible errors
   if(counted_bars<0) 
   {
      Print("if(counted_bars<0)");
   //   return(-1);
   }
   
// we must have at least slowEMA bars available:
   if ( Bars <= slowEMA ) return(0);   
      
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   
   limit=Bars-counted_bars ;
   
//---- main loop
   for(int i=limit-1; i>=0; i--) {
      //---- Start calculating DiNapoli MACD prediction:
      //---- First, we need current fast and slow EMA values:
      // (but standard iMA and iMACD require int values for periods, so we cannot use them)
      // So, we will use the custom DiNapoli MACD instead:
      // current_fastEMA = iMA(NULL,0,fastEMA,0,MODE_EMA,PRICE_CLOSE,i);
      
      current_fastEMA = iCustom(NULL, 0, "MACD (DEMA) DiNapoli", fastEMA, slowEMA, macdEMA, 2, i) ;  
                              
      // current_slowEMA = iMA(NULL,0,slowEMA,0,MODE_EMA,PRICE_CLOSE,i);
      current_slowEMA = iCustom(NULL, 0, "MACD (DEMA) DiNapoli", fastEMA, slowEMA, macdEMA, 3, i) ; 
                              
      //---- Second, calculate func currentEMAs:
      func_curr_EMA = ((1-gamma_fast)*current_fastEMA) - ((1-gamma_slow)*current_slowEMA) ;
      
      //---- Third, calculate func' current iMACD signal:
      // func_curr_sig = (1-gamma_macd)*iMACD(NULL,0,fastEMA,slowEMA,macdEMA,PRICE_CLOSE,MODE_SIGNAL,i) ;
      func_curr_sig = (1-gamma_macd)*iCustom(NULL, 0, "MACD (DEMA) DiNapoli", fastEMA, slowEMA, macdEMA, 1, i) ; 
                              
      //---- Finally, calculate the price level of future MACD/signal crossing:
      ExtRedBuffer[i] = ( -(func_curr_sig/(beta-1)) - func_curr_EMA )/alpha ;
   }
//---- done
   return(0);
  }
//+------------------------------------------------------------------+